@@ -27,9 +27,9 @@ int main()
     // interesting state.
     Myconv myconv;
     myconv.from_bytes("\xF1\x80\x80\x83");
-    const int old_converted = myconv.converted();
+    const auto old_converted = myconv.converted();
     assert(myconv.converted() == 4);
     // move construct a new converter and make sure the state is the same.
     Myconv myconv2(std::move(myconv));
-    assert(myconv2.converted() == 4);
+    assert(myconv2.converted() == old_converted);
 }
