@@ -15,17 +15,18 @@
 
 // This tests a conforming extension
 
+// UNSUPPORTED: c++98, c++03
+
 #include <queue>
 #include <cassert>
 
+#include "test_macros.h"
 #include "MoveOnly.h"
 
 int main()
 {
-#if __has_feature(cxx_noexcept)
     {
         typedef std::priority_queue<MoveOnly> C;
-        static_assert(std::is_nothrow_default_constructible<C>::value, "");
+        LIBCPP_STATIC_ASSERT(std::is_nothrow_default_constructible<C>::value, "");
     }
-#endif
 }
