@@ -23,7 +23,7 @@ int main()
         typedef int T;
         typedef std::forward_list<T> C;
         C c = {0, 1, 2, 3, 4, 5, 6, 7, 8, 9};
-        unsigned n = 0;
+        int n = 0;
         for (C::const_iterator i = c.begin(), e = c.end(); i != e; ++i, ++n)
             assert(*i == n);
         assert(n == 10);
@@ -33,7 +33,7 @@ int main()
         typedef int T;
         typedef std::forward_list<T, min_allocator<T>> C;
         C c = {0, 1, 2, 3, 4, 5, 6, 7, 8, 9};
-        unsigned n = 0;
+        int n = 0;
         for (C::const_iterator i = c.begin(), e = c.end(); i != e; ++i, ++n)
             assert(*i == n);
         assert(n == 10);
