@@ -17,7 +17,9 @@
 
 #include <unordered_set>
 #include <cassert>
+#include <cstddef>
 
+#include "test_macros.h"
 #include "../../../test_compare.h"
 #include "../../../test_hash.h"
 #include "test_allocator.h"
@@ -31,27 +33,27 @@ int main()
         typedef test_allocator<int> Alloc;
         typedef std::unordered_multiset<int, Hash, Compare, Alloc> C;
         typedef int P;
-        C c1(0, Hash(1), Compare(1), Alloc(1));
-        C c2(0, Hash(2), Compare(2), Alloc(2));
+        C c1(0, Hash(1), Compare(1), Alloc(1, 1));
+        C c2(0, Hash(2), Compare(2), Alloc(1, 2));
         c2.max_load_factor(2);
         swap(c1, c2);
 
-        assert(c1.bucket_count() == 0);
+        LIBCPP_ASSERT(c1.bucket_count() == 0);
         assert(c1.size() == 0);
         assert(c1.hash_function() == Hash(2));
         assert(c1.key_eq() == Compare(2));
-        assert(c1.get_allocator() == Alloc(1));
-        assert(std::distance(c1.begin(), c1.end()) == c1.size());
-        assert(std::distance(c1.cbegin(), c1.cend()) == c1.size());
+        assert(c1.get_allocator().get_id() == 1);
+        assert(static_cast<std::size_t>(std::distance(c1.begin(), c1.end())) == c1.size());
+        assert(static_cast<std::size_t>(std::distance(c1.cbegin(), c1.cend())) == c1.size());
         assert(c1.max_load_factor() == 2);
 
-        assert(c2.bucket_count() == 0);
+        LIBCPP_ASSERT(c2.bucket_count() == 0);
         assert(c2.size() == 0);
         assert(c2.hash_function() == Hash(1));
         assert(c2.key_eq() == Compare(1));
-        assert(c2.get_allocator() == Alloc(2));
-        assert(std::distance(c2.begin(), c2.end()) == c2.size());
-        assert(std::distance(c2.cbegin(), c2.cend()) == c2.size());
+        assert(c2.get_allocator().get_id() == 2);
+        assert(static_cast<std::size_t>(std::distance(c2.begin(), c2.end())) == c2.size());
+        assert(static_cast<std::size_t>(std::distance(c2.cbegin(), c2.cend())) == c2.size());
         assert(c2.max_load_factor() == 1);
     }
     {
@@ -71,12 +73,12 @@ int main()
             P(70),
             P(80)
         };
-        C c1(0, Hash(1), Compare(1), Alloc(1));
-        C c2(std::begin(a2), std::end(a2), 0, Hash(2), Compare(2), Alloc(2));
+        C c1(0, Hash(1), Compare(1), Alloc(1, 1));
+        C c2(std::begin(a2), std::end(a2), 0, Hash(2), Compare(2), Alloc(1, 2));
         c2.max_load_factor(2);
         swap(c1, c2);
 
-        assert(c1.bucket_count() >= 11);
+        assert(c1.bucket_count() >= 8);
         assert(c1.size() == 8);
         assert(*c1.find(10) == 10);
         assert(*c1.find(20) == 20);
@@ -88,18 +90,18 @@ int main()
         assert(*c1.find(80) == 80);
         assert(c1.hash_function() == Hash(2));
         assert(c1.key_eq() == Compare(2));
-        assert(c1.get_allocator() == Alloc(1));
-        assert(std::distance(c1.begin(), c1.end()) == c1.size());
-        assert(std::distance(c1.cbegin(), c1.cend()) == c1.size());
+        assert(c1.get_allocator().get_id() == 1);
+        assert(static_cast<std::size_t>(std::distance(c1.begin(), c1.end())) == c1.size());
+        assert(static_cast<std::size_t>(std::distance(c1.cbegin(), c1.cend())) == c1.size());
         assert(c1.max_load_factor() == 2);
 
-        assert(c2.bucket_count() == 0);
+        LIBCPP_ASSERT(c2.bucket_count() == 0);
         assert(c2.size() == 0);
         assert(c2.hash_function() == Hash(1));
         assert(c2.key_eq() == Compare(1));
-        assert(c2.get_allocator() == Alloc(2));
-        assert(std::distance(c2.begin(), c2.end()) == c2.size());
-        assert(std::distance(c2.cbegin(), c2.cend()) == c2.size());
+        assert(c2.get_allocator().get_id() == 2);
+        assert(static_cast<std::size_t>(std::distance(c2.begin(), c2.end())) == c2.size());
+        assert(static_cast<std::size_t>(std::distance(c2.cbegin(), c2.cend())) == c2.size());
         assert(c2.max_load_factor() == 1);
     }
     {
@@ -117,21 +119,21 @@ int main()
             P(1),
             P(2)
         };
-        C c1(std::begin(a1), std::end(a1), 0, Hash(1), Compare(1), Alloc(1));
-        C c2(0, Hash(2), Compare(2), Alloc(2));
+        C c1(std::begin(a1), std::end(a1), 0, Hash(1), Compare(1), Alloc(1, 1));
+        C c2(0, Hash(2), Compare(2), Alloc(1, 2));
         c2.max_load_factor(2);
         swap(c1, c2);
 
-        assert(c1.bucket_count() == 0);
+        LIBCPP_ASSERT(c1.bucket_count() == 0);
         assert(c1.size() == 0);
         assert(c1.hash_function() == Hash(2));
         assert(c1.key_eq() == Compare(2));
-        assert(c1.get_allocator() == Alloc(1));
-        assert(std::distance(c1.begin(), c1.end()) == c1.size());
-        assert(std::distance(c1.cbegin(), c1.cend()) == c1.size());
+        assert(c1.get_allocator().get_id() == 1);
+        assert(static_cast<std::size_t>(std::distance(c1.begin(), c1.end())) == c1.size());
+        assert(static_cast<std::size_t>(std::distance(c1.cbegin(), c1.cend())) == c1.size());
         assert(c1.max_load_factor() == 2);
 
-        assert(c2.bucket_count() >= 7);
+        assert(c2.bucket_count() >= 6);
         assert(c2.size() == 6);
         assert(c2.count(1) == 2);
         assert(c2.count(2) == 2);
@@ -139,9 +141,9 @@ int main()
         assert(c2.count(4) == 1);
         assert(c2.hash_function() == Hash(1));
         assert(c2.key_eq() == Compare(1));
-        assert(c2.get_allocator() == Alloc(2));
-        assert(std::distance(c2.begin(), c2.end()) == c2.size());
-        assert(std::distance(c2.cbegin(), c2.cend()) == c2.size());
+        assert(c2.get_allocator().get_id() == 2);
+        assert(static_cast<std::size_t>(std::distance(c2.begin(), c2.end())) == c2.size());
+        assert(static_cast<std::size_t>(std::distance(c2.cbegin(), c2.cend())) == c2.size());
         assert(c2.max_load_factor() == 1);
     }
     {
@@ -170,12 +172,12 @@ int main()
             P(70),
             P(80)
         };
-        C c1(std::begin(a1), std::end(a1), 0, Hash(1), Compare(1), Alloc(1));
-        C c2(std::begin(a2), std::end(a2), 0, Hash(2), Compare(2), Alloc(2));
+        C c1(std::begin(a1), std::end(a1), 0, Hash(1), Compare(1), Alloc(1, 1));
+        C c2(std::begin(a2), std::end(a2), 0, Hash(2), Compare(2), Alloc(1, 2));
         c2.max_load_factor(2);
         swap(c1, c2);
 
-        assert(c1.bucket_count() >= 11);
+        assert(c1.bucket_count() >= 8);
         assert(c1.size() == 8);
         assert(*c1.find(10) == 10);
         assert(*c1.find(20) == 20);
@@ -187,12 +189,12 @@ int main()
         assert(*c1.find(80) == 80);
         assert(c1.hash_function() == Hash(2));
         assert(c1.key_eq() == Compare(2));
-        assert(c1.get_allocator() == Alloc(1));
-        assert(std::distance(c1.begin(), c1.end()) == c1.size());
-        assert(std::distance(c1.cbegin(), c1.cend()) == c1.size());
+        assert(c1.get_allocator().get_id() == 1);
+        assert(static_cast<std::size_t>(std::distance(c1.begin(), c1.end())) == c1.size());
+        assert(static_cast<std::size_t>(std::distance(c1.cbegin(), c1.cend())) == c1.size());
         assert(c1.max_load_factor() == 2);
 
-        assert(c2.bucket_count() >= 7);
+        assert(c2.bucket_count() >= 6);
         assert(c2.size() == 6);
         assert(c2.count(1) == 2);
         assert(c2.count(2) == 2);
@@ -200,9 +202,9 @@ int main()
         assert(c2.count(4) == 1);
         assert(c2.hash_function() == Hash(1));
         assert(c2.key_eq() == Compare(1));
-        assert(c2.get_allocator() == Alloc(2));
-        assert(std::distance(c2.begin(), c2.end()) == c2.size());
-        assert(std::distance(c2.cbegin(), c2.cend()) == c2.size());
+        assert(c2.get_allocator().get_id() == 2);
+        assert(static_cast<std::size_t>(std::distance(c2.begin(), c2.end())) == c2.size());
+        assert(static_cast<std::size_t>(std::distance(c2.cbegin(), c2.cend())) == c2.size());
         assert(c2.max_load_factor() == 1);
     }
 
@@ -217,22 +219,22 @@ int main()
         c2.max_load_factor(2);
         swap(c1, c2);
 
-        assert(c1.bucket_count() == 0);
+        LIBCPP_ASSERT(c1.bucket_count() == 0);
         assert(c1.size() == 0);
         assert(c1.hash_function() == Hash(2));
         assert(c1.key_eq() == Compare(2));
         assert(c1.get_allocator() == Alloc(2));
-        assert(std::distance(c1.begin(), c1.end()) == c1.size());
-        assert(std::distance(c1.cbegin(), c1.cend()) == c1.size());
+        assert(static_cast<std::size_t>(std::distance(c1.begin(), c1.end())) == c1.size());
+        assert(static_cast<std::size_t>(std::distance(c1.cbegin(), c1.cend())) == c1.size());
         assert(c1.max_load_factor() == 2);
 
-        assert(c2.bucket_count() == 0);
+        LIBCPP_ASSERT(c2.bucket_count() == 0);
         assert(c2.size() == 0);
         assert(c2.hash_function() == Hash(1));
         assert(c2.key_eq() == Compare(1));
         assert(c2.get_allocator() == Alloc(1));
-        assert(std::distance(c2.begin(), c2.end()) == c2.size());
-        assert(std::distance(c2.cbegin(), c2.cend()) == c2.size());
+        assert(static_cast<std::size_t>(std::distance(c2.begin(), c2.end())) == c2.size());
+        assert(static_cast<std::size_t>(std::distance(c2.cbegin(), c2.cend())) == c2.size());
         assert(c2.max_load_factor() == 1);
     }
     {
@@ -257,7 +259,7 @@ int main()
         c2.max_load_factor(2);
         swap(c1, c2);
 
-        assert(c1.bucket_count() >= 11);
+        assert(c1.bucket_count() >= 8);
         assert(c1.size() == 8);
         assert(*c1.find(10) == 10);
         assert(*c1.find(20) == 20);
@@ -270,17 +272,17 @@ int main()
         assert(c1.hash_function() == Hash(2));
         assert(c1.key_eq() == Compare(2));
         assert(c1.get_allocator() == Alloc(2));
-        assert(std::distance(c1.begin(), c1.end()) == c1.size());
-        assert(std::distance(c1.cbegin(), c1.cend()) == c1.size());
+        assert(static_cast<std::size_t>(std::distance(c1.begin(), c1.end())) == c1.size());
+        assert(static_cast<std::size_t>(std::distance(c1.cbegin(), c1.cend())) == c1.size());
         assert(c1.max_load_factor() == 2);
 
-        assert(c2.bucket_count() == 0);
+        LIBCPP_ASSERT(c2.bucket_count() == 0);
         assert(c2.size() == 0);
         assert(c2.hash_function() == Hash(1));
         assert(c2.key_eq() == Compare(1));
         assert(c2.get_allocator() == Alloc(1));
-        assert(std::distance(c2.begin(), c2.end()) == c2.size());
-        assert(std::distance(c2.cbegin(), c2.cend()) == c2.size());
+        assert(static_cast<std::size_t>(std::distance(c2.begin(), c2.end())) == c2.size());
+        assert(static_cast<std::size_t>(std::distance(c2.cbegin(), c2.cend())) == c2.size());
         assert(c2.max_load_factor() == 1);
     }
     {
@@ -303,16 +305,16 @@ int main()
         c2.max_load_factor(2);
         swap(c1, c2);
 
-        assert(c1.bucket_count() == 0);
+        LIBCPP_ASSERT(c1.bucket_count() == 0);
         assert(c1.size() == 0);
         assert(c1.hash_function() == Hash(2));
         assert(c1.key_eq() == Compare(2));
         assert(c1.get_allocator() == Alloc(2));
-        assert(std::distance(c1.begin(), c1.end()) == c1.size());
-        assert(std::distance(c1.cbegin(), c1.cend()) == c1.size());
+        assert(static_cast<std::size_t>(std::distance(c1.begin(), c1.end())) == c1.size());
+        assert(static_cast<std::size_t>(std::distance(c1.cbegin(), c1.cend())) == c1.size());
         assert(c1.max_load_factor() == 2);
 
-        assert(c2.bucket_count() >= 7);
+        assert(c2.bucket_count() >= 6);
         assert(c2.size() == 6);
         assert(c2.count(1) == 2);
         assert(c2.count(2) == 2);
@@ -321,8 +323,8 @@ int main()
         assert(c2.hash_function() == Hash(1));
         assert(c2.key_eq() == Compare(1));
         assert(c2.get_allocator() == Alloc(1));
-        assert(std::distance(c2.begin(), c2.end()) == c2.size());
-        assert(std::distance(c2.cbegin(), c2.cend()) == c2.size());
+        assert(static_cast<std::size_t>(std::distance(c2.begin(), c2.end())) == c2.size());
+        assert(static_cast<std::size_t>(std::distance(c2.cbegin(), c2.cend())) == c2.size());
         assert(c2.max_load_factor() == 1);
     }
     {
@@ -356,7 +358,7 @@ int main()
         c2.max_load_factor(2);
         swap(c1, c2);
 
-        assert(c1.bucket_count() >= 11);
+        assert(c1.bucket_count() >= 8);
         assert(c1.size() == 8);
         assert(*c1.find(10) == 10);
         assert(*c1.find(20) == 20);
@@ -369,11 +371,11 @@ int main()
         assert(c1.hash_function() == Hash(2));
         assert(c1.key_eq() == Compare(2));
         assert(c1.get_allocator() == Alloc(2));
-        assert(std::distance(c1.begin(), c1.end()) == c1.size());
-        assert(std::distance(c1.cbegin(), c1.cend()) == c1.size());
+        assert(static_cast<std::size_t>(std::distance(c1.begin(), c1.end())) == c1.size());
+        assert(static_cast<std::size_t>(std::distance(c1.cbegin(), c1.cend())) == c1.size());
         assert(c1.max_load_factor() == 2);
 
-        assert(c2.bucket_count() >= 7);
+        assert(c2.bucket_count() >= 6);
         assert(c2.size() == 6);
         assert(c2.count(1) == 2);
         assert(c2.count(2) == 2);
@@ -382,8 +384,8 @@ int main()
         assert(c2.hash_function() == Hash(1));
         assert(c2.key_eq() == Compare(1));
         assert(c2.get_allocator() == Alloc(1));
-        assert(std::distance(c2.begin(), c2.end()) == c2.size());
-        assert(std::distance(c2.cbegin(), c2.cend()) == c2.size());
+        assert(static_cast<std::size_t>(std::distance(c2.begin(), c2.end())) == c2.size());
+        assert(static_cast<std::size_t>(std::distance(c2.cbegin(), c2.cend())) == c2.size());
         assert(c2.max_load_factor() == 1);
     }
 #if TEST_STD_VER >= 11
@@ -398,22 +400,22 @@ int main()
         c2.max_load_factor(2);
         swap(c1, c2);
 
-        assert(c1.bucket_count() == 0);
+        LIBCPP_ASSERT(c1.bucket_count() == 0);
         assert(c1.size() == 0);
         assert(c1.hash_function() == Hash(2));
         assert(c1.key_eq() == Compare(2));
         assert(c1.get_allocator() == Alloc());
-        assert(std::distance(c1.begin(), c1.end()) == c1.size());
-        assert(std::distance(c1.cbegin(), c1.cend()) == c1.size());
+        assert(static_cast<std::size_t>(std::distance(c1.begin(), c1.end())) == c1.size());
+        assert(static_cast<std::size_t>(std::distance(c1.cbegin(), c1.cend())) == c1.size());
         assert(c1.max_load_factor() == 2);
 
-        assert(c2.bucket_count() == 0);
+        LIBCPP_ASSERT(c2.bucket_count() == 0);
         assert(c2.size() == 0);
         assert(c2.hash_function() == Hash(1));
         assert(c2.key_eq() == Compare(1));
         assert(c2.get_allocator() == Alloc());
-        assert(std::distance(c2.begin(), c2.end()) == c2.size());
-        assert(std::distance(c2.cbegin(), c2.cend()) == c2.size());
+        assert(static_cast<std::size_t>(std::distance(c2.begin(), c2.end())) == c2.size());
+        assert(static_cast<std::size_t>(std::distance(c2.cbegin(), c2.cend())) == c2.size());
         assert(c2.max_load_factor() == 1);
     }
     {
@@ -438,7 +440,7 @@ int main()
         c2.max_load_factor(2);
         swap(c1, c2);
 
-        assert(c1.bucket_count() >= 11);
+        assert(c1.bucket_count() >= 8);
         assert(c1.size() == 8);
         assert(*c1.find(10) == 10);
         assert(*c1.find(20) == 20);
@@ -451,17 +453,17 @@ int main()
         assert(c1.hash_function() == Hash(2));
         assert(c1.key_eq() == Compare(2));
         assert(c1.get_allocator() == Alloc());
-        assert(std::distance(c1.begin(), c1.end()) == c1.size());
-        assert(std::distance(c1.cbegin(), c1.cend()) == c1.size());
+        assert(static_cast<std::size_t>(std::distance(c1.begin(), c1.end())) == c1.size());
+        assert(static_cast<std::size_t>(std::distance(c1.cbegin(), c1.cend())) == c1.size());
         assert(c1.max_load_factor() == 2);
 
-        assert(c2.bucket_count() == 0);
+        LIBCPP_ASSERT(c2.bucket_count() == 0);
         assert(c2.size() == 0);
         assert(c2.hash_function() == Hash(1));
         assert(c2.key_eq() == Compare(1));
         assert(c2.get_allocator() == Alloc());
-        assert(std::distance(c2.begin(), c2.end()) == c2.size());
-        assert(std::distance(c2.cbegin(), c2.cend()) == c2.size());
+        assert(static_cast<std::size_t>(std::distance(c2.begin(), c2.end())) == c2.size());
+        assert(static_cast<std::size_t>(std::distance(c2.cbegin(), c2.cend())) == c2.size());
         assert(c2.max_load_factor() == 1);
     }
     {
@@ -484,16 +486,16 @@ int main()
         c2.max_load_factor(2);
         swap(c1, c2);
 
-        assert(c1.bucket_count() == 0);
+        LIBCPP_ASSERT(c1.bucket_count() == 0);
         assert(c1.size() == 0);
         assert(c1.hash_function() == Hash(2));
         assert(c1.key_eq() == Compare(2));
         assert(c1.get_allocator() == Alloc());
-        assert(std::distance(c1.begin(), c1.end()) == c1.size());
-        assert(std::distance(c1.cbegin(), c1.cend()) == c1.size());
+        assert(static_cast<std::size_t>(std::distance(c1.begin(), c1.end())) == c1.size());
+        assert(static_cast<std::size_t>(std::distance(c1.cbegin(), c1.cend())) == c1.size());
         assert(c1.max_load_factor() == 2);
 
-        assert(c2.bucket_count() >= 7);
+        assert(c2.bucket_count() >= 6);
         assert(c2.size() == 6);
         assert(c2.count(1) == 2);
         assert(c2.count(2) == 2);
@@ -502,8 +504,8 @@ int main()
         assert(c2.hash_function() == Hash(1));
         assert(c2.key_eq() == Compare(1));
         assert(c2.get_allocator() == Alloc());
-        assert(std::distance(c2.begin(), c2.end()) == c2.size());
-        assert(std::distance(c2.cbegin(), c2.cend()) == c2.size());
+        assert(static_cast<std::size_t>(std::distance(c2.begin(), c2.end())) == c2.size());
+        assert(static_cast<std::size_t>(std::distance(c2.cbegin(), c2.cend())) == c2.size());
         assert(c2.max_load_factor() == 1);
     }
     {
@@ -537,7 +539,7 @@ int main()
         c2.max_load_factor(2);
         swap(c1, c2);
 
-        assert(c1.bucket_count() >= 11);
+        assert(c1.bucket_count() >= 8);
         assert(c1.size() == 8);
         assert(*c1.find(10) == 10);
         assert(*c1.find(20) == 20);
@@ -550,11 +552,11 @@ int main()
         assert(c1.hash_function() == Hash(2));
         assert(c1.key_eq() == Compare(2));
         assert(c1.get_allocator() == Alloc());
-        assert(std::distance(c1.begin(), c1.end()) == c1.size());
-        assert(std::distance(c1.cbegin(), c1.cend()) == c1.size());
+        assert(static_cast<std::size_t>(std::distance(c1.begin(), c1.end())) == c1.size());
+        assert(static_cast<std::size_t>(std::distance(c1.cbegin(), c1.cend())) == c1.size());
         assert(c1.max_load_factor() == 2);
 
-        assert(c2.bucket_count() >= 7);
+        assert(c2.bucket_count() >= 6);
         assert(c2.size() == 6);
         assert(c2.count(1) == 2);
         assert(c2.count(2) == 2);
@@ -563,8 +565,8 @@ int main()
         assert(c2.hash_function() == Hash(1));
         assert(c2.key_eq() == Compare(1));
         assert(c2.get_allocator() == Alloc());
-        assert(std::distance(c2.begin(), c2.end()) == c2.size());
-        assert(std::distance(c2.cbegin(), c2.cend()) == c2.size());
+        assert(static_cast<std::size_t>(std::distance(c2.begin(), c2.end())) == c2.size());
+        assert(static_cast<std::size_t>(std::distance(c2.cbegin(), c2.cend())) == c2.size());
         assert(c2.max_load_factor() == 1);
     }
 #endif
