@@ -35,11 +35,11 @@ test()
 
     for (int i = 0; i <= 5; ++i)
     {
-        T t(i);
+        T t(static_cast<T>(i));
         if (sizeof(T) <= sizeof(std::size_t))
         {
             const std::size_t result = h(t);
-            LIBCPP_ASSERT(result == t);
+            LIBCPP_ASSERT(result == static_cast<size_t>(t));
             ((void)result); // Prevent unused warning
         }
     }
