@@ -25,11 +25,16 @@ def main():
     parser.add_argument('--names-only', dest='names_only',
                         help='Output only the name of the symbol',
                         action='store_true', default=False)
+    parser.add_argument('--only-stdlib-symbols', dest='only_stdlib',
+                        help="Filter all symbols not related to the stdlib",
+                        action='store_true', default=False)
     args = parser.parse_args()
     if args.output is not None:
         print('Extracting symbols from %s to %s.'
               % (args.library, args.output))
     syms = extract.extract_symbols(args.library)
+    if args.only_stdlib:
+        syms, other_syms = util.filter_stdlib_symbols(syms)
     util.write_syms(syms, out=args.output, names_only=args.names_only)
 
 
